using System;
using System.Collections.Generic;
using System.Text;
using System.Threading; 

namespace Monitoring
{
    /// <summary>
    /// A class that allows an externally accessible hearbeat to be set for an application 
    /// </summary>
    public static class Heartbeat  
    {

        private static HeartbeatListenerCollection _listeners = new HeartbeatListenerCollection();
        private static AutoResetEvent autoEvent;
        private static System.Threading.Timer heartbeatTimer;

        /// <summary>
        /// Reads the heartbeat listensers collection
        /// from the application configuration
        /// </summary>
        /// <remarks >
        /// If no listeners are set a default TraceHeartbeatMonitor is added
        /// </remarks>
        public static void ResetListensersFromConfig()
        {
            HeartbeatCustomConfigSection _config = System.Configuration.ConfigurationManager.GetSection("heartbeatSettings") as HeartbeatCustomConfigSection;

            foreach (HeartbeatCustomConfigElement  _element in _config.Heartbeats )
            {
                // Add the listensers if they aren't already added
                if (!(_listeners.ContainsName(_element.Name )))
                {
                    _listeners.Add(_element.Instantiate()); 
                }
            }

            if (_listeners.Count == 0)
            {
                // Add a TraceHeartbeatMonitor by default
                TraceHeartbeatMonitor _defaultListener = new TraceHeartbeatMonitor();
                _defaultListener.Name = "DefaultHeartbeatTrace";
                _listeners.Add(_defaultListener);
            }
        }

        /// <summary>
        /// Sends a heartbeat to any heartbeat monitors
        /// to indicate that the application is running
        /// </summary>
        public static void Beat()
        {
            // make each heartbeat listener beat 
            foreach (HeartbeatMonitorBase  _listener in Listeners)
            {
                _listener.Beat();
            }
        }

        // The timer-called Beat 
        public static void Beat(object beatData)
        {
            Heartbeat.Beat();
        }

        /// <summary>
        /// Sends a heartbeat to any heartbeat monitors
        /// to indicate that the application is running
        /// </summary>
        /// <param name="BeatValue" >
        /// The value to pass extra information to the listeners 
        /// </param>
        public static void Beat(int BeatValue)
        {
            // make each heartbeat listener beat 
            foreach (HeartbeatMonitorBase _listener in Listeners)
            {
                _listener.Beat(BeatValue );
            }
        }

        /// <summary>
        /// Starts the heartbeat with a default frequency
        /// </summary>
        static Heartbeat()
        {

            // Fire up a thread to invoke hearbeat events 
            // perdiodically...
            TimerCallback timerDelegate =
            new TimerCallback(Heartbeat.Beat);

            autoEvent = new AutoResetEvent(false);

            // Start the heartbeat going
            heartbeatTimer =
                new System.Threading.Timer(timerDelegate, autoEvent, 0,1000);
            

        }


        /// <summary>
        /// The listeners attached to the heartbeat
        /// </summary>
        /// <remarks >
        /// These should not be triggered directly but rather the 
        /// Heartbeat.Beat() method should be called
        /// </remarks>
        public  static HeartbeatListenerCollection Listeners
        {
            get { return Heartbeat._listeners; }
        }

    }

}
