using System;
using System.Configuration;

namespace Monitoring
{
    /// <summary>
    /// The class that reads the "heartbeats" section of the application configuration file
    /// in order to attach heartbeat listeners to the application
    /// </summary>
    public sealed class HeartbeatCustomConfigSection : ConfigurationSection
    {

        #region Private members
        private static ConfigurationPropertyCollection m_properties;
        private static ConfigurationProperty s_propCollection;
        #endregion

        [ConfigurationProperty("heartbeats", IsDefaultCollection=true )]
        public HeartbeatConfigurationElementCollection Heartbeats
        {
            get
            {
                return ((HeartbeatConfigurationElementCollection)this["heartbeats"]);
            }
        }


        /// <summary>
        /// Returns the properties for this configuration collection
        /// </summary>
        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return m_properties;
            }
        }

        public override bool IsReadOnly()
        {
            return false;
        }

        #region Constructors
        static HeartbeatCustomConfigSection()
        {

            s_propCollection = new ConfigurationProperty("heartbeats",
                                                         typeof(HeartbeatConfigurationElementCollection),
                                                         null,
                                                         ConfigurationPropertyOptions.IsDefaultCollection); 

            m_properties = new ConfigurationPropertyCollection();
            m_properties.Add(s_propCollection );
        }

        public HeartbeatCustomConfigSection()
        {
        }

        #endregion

    }

    [ConfigurationCollection(typeof(HeartbeatCustomConfigElement),
        CollectionType = ConfigurationElementCollectionType.AddRemoveClearMap)]
    public sealed class HeartbeatConfigurationElementCollection : ConfigurationElementCollection
    {

        #region Private members
        private static ConfigurationPropertyCollection m_properties;
        #endregion
        
        /// <summary>
        /// Tells the custom configuration reader what kind of collection the 
        /// configuration section contains
        /// </summary>
        /// <remarks >
        /// In this instance we want &lt;clear/&gt; , &lt;add/&gt; and 
        /// &lt;remove/&gt; elements
        /// </remarks>
        public override  ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.AddRemoveClearMap;
            }
        }

        /// <summary>
        /// Gets the key for a given HeartbeatConfigurationElememnt
        /// </summary>
        /// <param name="element"></param>
        /// <returns>
        /// The Name of the heartbeat configuration element
        /// </returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((HeartbeatCustomConfigElement)element).Name; 
        }




        /// <summary>
        /// Adds the heartbeat listener identified by the heartbeat custom
        /// config element
        /// </summary>
        /// <param name="heartbeatListener"></param>
        public void add(HeartbeatCustomConfigElement heartbeatListener)
        {
            // Add 
            BaseAdd(heartbeatListener); 
            // and add the listener to the Hearbeat listensers collection
            HeartbeatMonitorBase _tmp = heartbeatListener.Instantiate();
            if (_tmp != null)
            {
                Heartbeat.Listeners.Add(_tmp);   
            }
        }

        /// <summary>
        /// Removes teh named listener from the heartbeat listeners collection
        /// </summary>
        /// <param name="name"></param>
        public void Remove(string name)
        {
            BaseRemove(name);
        }

        /// <summary>
        /// Clear the heartbeat listeners collection
        /// </summary>
        public void clear()
        {
            Heartbeat.Listeners.Clear();
            BaseClear();
        }

        public override bool IsReadOnly()
        {
            return false;
        }

        /// <summary>
        /// Creates a new heartbeat custom configuration
        /// elements
        /// </summary>
        /// <returns></returns>
        protected  override ConfigurationElement CreateNewElement()
        {
            return new HeartbeatCustomConfigElement();
        }

        /// <summary>
        /// Returns the properties for this configuration collection
        /// </summary>
        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return m_properties ;
            }
        }

        #region Indexers
        public HeartbeatCustomConfigElement this[int index]
        {
            get { return (HeartbeatCustomConfigElement)base.BaseGet(index); }
            set
            {
                if (base.BaseGet(index) != null)
                {
                    base.BaseRemoveAt(index);
                }
                base.BaseAdd(index, value);
            }
        }

        public HeartbeatCustomConfigElement this[string name]
        {
            get { return (HeartbeatCustomConfigElement)base.BaseGet(name); }
        }

        #endregion

        #region Constructors
        static HeartbeatConfigurationElementCollection()
        {
        m_properties = new ConfigurationPropertyCollection();
        }

        public HeartbeatConfigurationElementCollection()
        {
        }
        #endregion

}

    /// <summary>
    /// Class that represents a single listener item in the listeners collection
    /// attached to the Heartbeat class
    /// </summary>
    public sealed class HeartbeatCustomConfigElement : ConfigurationElement
    {

        #region Static Fields
        private static ConfigurationProperty s_propName;
        private static ConfigurationProperty s_propType;
        private static ConfigurationProperty s_propInitdata;

        private static ConfigurationPropertyCollection s_properties;
        #endregion


        /// <summary>
        /// The unique name of this heartbeat listener
        /// </summary>
        /// <remarks >
        /// Listeners are indexed and removed by name so this value must be unique
        /// within the HeartbeatsCustomConfigSection
        /// </remarks>
        [ConfigurationProperty("Name", IsRequired = true, IsKey = true)]
        public string Name
        {
            get
            {
                return ((string)base[s_propName] );
            }
            set
            {
                base[s_propName]  = value;
            }
        }

        [ConfigurationProperty("Type", IsRequired = true)]
        public string TypeName
        {
            get
            {
                return ((string)base[s_propType]);
            }
            set
            {
                base[s_propType] = value;
            }
        }

        /// <summary>
        /// Initialisation data for the heartbeat listener
        /// </summary>
        [ConfigurationProperty("InitData", IsRequired = false)]
        public string InitData
        {
            get
            {
                return ((string)base[s_propInitdata]);
            }
            set
            {
                base[s_propInitdata] = value;
            }
        }

        /// <summary>
        /// Creates a new instance of the heartbeat monitor base class
        /// that this heartbeat configuration element defines
        /// </summary>
        /// <returns>
        /// An instance of a class that inherits from HeartbeatMonitorBase
        /// </returns>
        public HeartbeatMonitorBase Instantiate()
        {
            Type heartbeatMonitorType = Type.GetType(this.TypeName);
            if (heartbeatMonitorType != null)
            {
                if (base[s_propInitdata] == null)
                {
                    System.Reflection.ConstructorInfo _heartbeatConstructor = heartbeatMonitorType.GetConstructor(Type.EmptyTypes);
                    if (_heartbeatConstructor != null)
                    {
                        return ((HeartbeatMonitorBase)_heartbeatConstructor.Invoke(Type.EmptyTypes));
                    }
                    else
                    {
                        throw new ConfigurationErrorsException("Heartbeat monitor does not have a constructor with zero parameters");
                    }
                }
                else
                {
                    // Get the constructor that takes a string
                    Type[] types = new Type[1];
                    types[0] = typeof(string);
                    System.Reflection.ConstructorInfo _heartbeatConstructor = heartbeatMonitorType.GetConstructor(types);
                    if (_heartbeatConstructor != null)
                    {
                        string _initData = (string)base[s_propInitdata];
                        return ((HeartbeatMonitorBase)_heartbeatConstructor.Invoke(new object[] {_initData}));
                    }
                    else
                    {
                        throw new ConfigurationErrorsException("Heartbeat monitor does not have a constructor with zero parameters");
                    }
                }
            }
            else
            {
                throw new ConfigurationErrorsException("Heartbeat monitor type could not be created");
            }
        }


        protected override ConfigurationPropertyCollection Properties
        {
            get { return s_properties; }
        }

        public override bool IsReadOnly()
        {
            return false;
        }

        #region Public constructors

        static HeartbeatCustomConfigElement()
        {
            s_propName = new ConfigurationProperty(
                "Name",
                typeof(string),
                null,
                ConfigurationPropertyOptions.IsRequired
            );

            s_propType = new ConfigurationProperty(
                "Type",
                typeof(string),
                null,
                ConfigurationPropertyOptions.IsRequired
            );

            s_propInitdata = new ConfigurationProperty(
                 "InitData",
                 typeof(string),
                 null,
                 ConfigurationPropertyOptions.None
                 );

            s_properties = new ConfigurationPropertyCollection();

            s_properties.Add(s_propName);
            s_properties.Add(s_propType);
            s_properties.Add(s_propInitdata);

        }

        public HeartbeatCustomConfigElement()
        {
        }
        public HeartbeatCustomConfigElement(string name,
                                            string typeName,
                                            string initData)
        {
            Name = name;
            TypeName = typeName;
            InitData = initData;
        }
        #endregion

    }
}
